# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Utilities for running protoc, and easily composing Protobuf calls in python.

To create a Protobuf plugin in Python, define an entrypoint as so:

.. code-block:: python

   @protoc_plugin
   def main(request: CodeGeneratorRequest) -> CodeGeneratorResponse:
       # This does what `python3 -m grpc_tools.protoc ...` does,
       # but as a function call in Python instead of an awkward exec
       python_response = run_plugin('python', request)

       # It is typical for a plugin to need to ultimately call one of the
       # built-in plugins (for example, to post-process the data from a built-in plugin).
       for f in python_response.file:
           f.proto_file = COPYRIGHT_NOTICE + f.proto_file

       response = CodeGeneratorResponse()
       response.MergeFrom(python_response)
       return response
"""

from __future__ import annotations

from importlib import resources
from functools import wraps
import os
from pathlib import Path
import site
import subprocess
import sys
from tempfile import TemporaryDirectory
from traceback import print_exc
from typing import Callable, List, NoReturn

from google.protobuf.compiler.plugin_pb2 import CodeGeneratorRequest, CodeGeneratorResponse
from google.protobuf.descriptor_pb2 import FileDescriptorSet
from grpc_tools import protoc as _protoc

__all__ = ["main", "protoc_plugin", "run_plugin"]


def main() -> NoReturn:
    """
    Entrypoint that runs ``protoc``, but with locally exported binary plugins, and automatically
    including Protobuf files that are built into ``grpc_tools`` and that have been pulled through
    as Python dependencies.
    """
    sys.exit(_main(sys.argv))


def _main(argv):
    os.environ["PATH"] = f'{os.getenv("PATH")}:{os.path.dirname(sys.executable)}'

    with resources.path("grpc_tools", "_proto") as proto_includes:
        includes = [str(proto_includes), *site.getsitepackages()]
        return _protoc.main(argv + ["-I" + inc for inc in includes])


def protoc_plugin(
    fn: Callable[[CodeGeneratorRequest], CodeGeneratorResponse],
) -> Callable[[], NoReturn]:
    """
    Decorator that converts a callable function to a ``protoc`` plugin.

    This function does NOT return, and is intended to be used as the main function for a Python
    program/script.
    """

    @wraps(fn)
    def _body() -> NoReturn:
        try:
            data = sys.stdin.buffer.read()
            request = CodeGeneratorRequest.FromString(data)
            response = fn(request)
            code = 0
        except Exception as ex:
            print_exc(file=sys.stderr)
            response = CodeGeneratorResponse(error=str(ex))
            code = 1

        sys.stdout.buffer.write(response.SerializeToString())
        sys.exit(code)

    return _body


def run_plugin(plugin_name: str, request: CodeGeneratorRequest) -> CodeGeneratorResponse:
    if plugin_name in ("python", "grpc_python"):
        return run_plugin_built_in(plugin_name, request)
    else:
        return run_plugin_external(plugin_name, request)


def run_plugin_built_in(plugin_name: str, request: CodeGeneratorRequest) -> CodeGeneratorResponse:
    with TemporaryDirectory() as tmpdir:
        input_file = Path(tmpdir) / "in.bin"
        output_dir = Path(tmpdir) / "out"

        output_dir.mkdir()

        input_file.write_bytes(FileDescriptorSet(file=request.proto_file).SerializeToString())

        # re-run the built-in code generator, but with the arguments that we were given
        invocation = [
            "_dazl_pb.protoc",
            f"--descriptor_set_in={input_file}",
            f"--{plugin_name}_out={output_dir}",
            *request.file_to_generate,
        ]

        exit_code = _main(invocation)
        if exit_code:
            raise Exception(f"failed to run {invocation} (exit code {exit_code})")

        files = [
            CodeGeneratorResponse.File(name=str(p.relative_to(output_dir)), content=p.read_text())
            for p in output_dir.rglob("*")
            if p.is_file()
        ]
        return CodeGeneratorResponse(file=files)


def run_plugin_external(plugin_name: str, request: CodeGeneratorRequest) -> CodeGeneratorResponse:
    args = ["protoc-gen-" + plugin_name]  # type: List[str]
    proc = subprocess.run(args, input=request.SerializeToString(), capture_output=True)  # type: ignore

    response = CodeGeneratorResponse()
    response.ParseFromString(proc.stdout)
    return response



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.duration_pb2 import Duration
from google.protobuf.message import Message as _Message

__all__ = [
    "GetLedgerConfigurationRequest",
    "GetLedgerConfigurationResponse",
    "LedgerConfiguration",
]


class GetLedgerConfigurationRequest(_Message):
    ledger_id: _builtins.str
    def __init__(self, *, ledger_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["ledger_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["ledger_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetLedgerConfigurationResponse(_Message):
    @property
    def ledger_configuration(self) -> LedgerConfiguration: ...
    def __init__(self, *, ledger_configuration: _typing.Optional[LedgerConfiguration] = ...): ...
    def HasField(self, field_name: _typing.Literal["ledger_configuration"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["ledger_configuration"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class LedgerConfiguration(_Message):
    @property
    def max_deduplication_duration(self) -> Duration: ...
    def __init__(self, *, max_deduplication_duration: _typing.Optional[Duration] = ...): ...
    def HasField(self, field_name: _typing.Literal["max_deduplication_duration"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["max_deduplication_duration"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.field_mask_pb2 import FieldMask
from google.protobuf.internal.containers import RepeatedCompositeFieldContainer
from google.protobuf.message import Message as _Message

__all__ = [
    "IdentityProviderConfig",
    "CreateIdentityProviderConfigRequest",
    "CreateIdentityProviderConfigResponse",
    "GetIdentityProviderConfigRequest",
    "GetIdentityProviderConfigResponse",
    "ListIdentityProviderConfigsRequest",
    "ListIdentityProviderConfigsResponse",
    "UpdateIdentityProviderConfigRequest",
    "UpdateIdentityProviderConfigResponse",
    "DeleteIdentityProviderConfigRequest",
    "DeleteIdentityProviderConfigResponse",
]


class IdentityProviderConfig(_Message):
    identity_provider_id: _builtins.str
    is_deactivated: _builtins.bool
    issuer: _builtins.str
    jwks_url: _builtins.str
    def __init__(self, *, identity_provider_id: _typing.Optional[_builtins.str] = ..., is_deactivated: _typing.Optional[_builtins.bool] = ..., issuer: _typing.Optional[_builtins.str] = ..., jwks_url: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["identity_provider_id", "is_deactivated", "issuer", "jwks_url"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["identity_provider_id", "is_deactivated", "issuer", "jwks_url"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class CreateIdentityProviderConfigRequest(_Message):
    @property
    def identity_provider_config(self) -> IdentityProviderConfig: ...
    def __init__(self, *, identity_provider_config: _typing.Optional[IdentityProviderConfig] = ...): ...
    def HasField(self, field_name: _typing.Literal["identity_provider_config"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["identity_provider_config"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class CreateIdentityProviderConfigResponse(_Message):
    @property
    def identity_provider_config(self) -> IdentityProviderConfig: ...
    def __init__(self, *, identity_provider_config: _typing.Optional[IdentityProviderConfig] = ...): ...
    def HasField(self, field_name: _typing.Literal["identity_provider_config"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["identity_provider_config"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetIdentityProviderConfigRequest(_Message):
    identity_provider_id: _builtins.str
    def __init__(self, *, identity_provider_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["identity_provider_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["identity_provider_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetIdentityProviderConfigResponse(_Message):
    @property
    def identity_provider_config(self) -> IdentityProviderConfig: ...
    def __init__(self, *, identity_provider_config: _typing.Optional[IdentityProviderConfig] = ...): ...
    def HasField(self, field_name: _typing.Literal["identity_provider_config"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["identity_provider_config"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class ListIdentityProviderConfigsRequest(_Message):
    def __init__(self): ...
    def HasField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def ClearField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class ListIdentityProviderConfigsResponse(_Message):
    @property
    def identity_provider_configs(self) -> RepeatedCompositeFieldContainer[IdentityProviderConfig]: ...
    def __init__(self, *, identity_provider_configs: _typing.Optional[_typing.Iterable[IdentityProviderConfig]] = ...): ...
    def HasField(self, field_name: _typing.Literal["identity_provider_configs"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["identity_provider_configs"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class UpdateIdentityProviderConfigRequest(_Message):
    @property
    def identity_provider_config(self) -> IdentityProviderConfig: ...
    @property
    def update_mask(self) -> FieldMask: ...
    def __init__(self, *, identity_provider_config: _typing.Optional[IdentityProviderConfig] = ..., update_mask: _typing.Optional[FieldMask] = ...): ...
    def HasField(self, field_name: _typing.Literal["identity_provider_config", "update_mask"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["identity_provider_config", "update_mask"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class UpdateIdentityProviderConfigResponse(_Message):
    @property
    def identity_provider_config(self) -> IdentityProviderConfig: ...
    def __init__(self, *, identity_provider_config: _typing.Optional[IdentityProviderConfig] = ...): ...
    def HasField(self, field_name: _typing.Literal["identity_provider_config"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["identity_provider_config"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class DeleteIdentityProviderConfigRequest(_Message):
    identity_provider_id: _builtins.str
    def __init__(self, *, identity_provider_id: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["identity_provider_id"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["identity_provider_id"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class DeleteIdentityProviderConfigResponse(_Message):
    def __init__(self): ...
    def HasField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def ClearField(self, field_name: _typing.NoReturn) -> _typing.NoReturn: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.internal.containers import MessageMap, RepeatedCompositeFieldContainer
from google.protobuf.message import Message as _Message

from .value_pb2 import Identifier

__all__ = [
    "TransactionFilter",
    "Filters",
    "InclusiveFilters",
    "InterfaceFilter",
]


class TransactionFilter(_Message):
    @property
    def filters_by_party(self) -> MessageMap[_builtins.str, Filters]: ...
    def __init__(self, *, filters_by_party: _typing.Optional[_typing.Mapping[_builtins.str, Filters]] = ...): ...
    def HasField(self, field_name: _typing.Literal["filters_by_party"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["filters_by_party"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class Filters(_Message):
    @property
    def inclusive(self) -> InclusiveFilters: ...
    def __init__(self, *, inclusive: _typing.Optional[InclusiveFilters] = ...): ...
    def HasField(self, field_name: _typing.Literal["inclusive"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["inclusive"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class InclusiveFilters(_Message):
    @property
    def template_ids(self) -> RepeatedCompositeFieldContainer[Identifier]: ...
    @property
    def interface_filters(self) -> RepeatedCompositeFieldContainer[InterfaceFilter]: ...
    def __init__(self, *, template_ids: _typing.Optional[_typing.Iterable[Identifier]] = ..., interface_filters: _typing.Optional[_typing.Iterable[InterfaceFilter]] = ...): ...
    def HasField(self, field_name: _typing.Literal["template_ids", "interface_filters"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["template_ids", "interface_filters"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class InterfaceFilter(_Message):
    @property
    def interface_id(self) -> Identifier: ...
    include_interface_view: _builtins.bool
    include_create_arguments_blob: _builtins.bool
    def __init__(self, *, interface_id: _typing.Optional[Identifier] = ..., include_interface_view: _typing.Optional[_builtins.bool] = ..., include_create_arguments_blob: _typing.Optional[_builtins.bool] = ...): ...
    def HasField(self, field_name: _typing.Literal["interface_id", "include_interface_view", "include_create_arguments_blob"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["interface_id", "include_interface_view", "include_create_arguments_blob"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module contains helper utilities for working within the constraints of Python's type system.
"""

from __future__ import annotations

from typing import Any, Type, TypeVar

__all__ = ["safe_cast"]

K = TypeVar("K")
V = TypeVar("V")
T = TypeVar("T")


def safe_cast(typ: Type[T], val: Any) -> T:
    """
    Safely casts an item to the specified type. Unlike ``typing.cast``, this method will actually
    throw an exception if types do not match.
    """
    if isinstance(typ, type):
        if isinstance(val, typ):
            return val
        else:
            raise ValueError(f"expected a value of type {typ} here (got {val} instead)")
    else:
        raise Exception(f"safe_cast can only be used with actual types (got {typ})")



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module contains miscellaneous utility methods that don't really fit anywhere else.
"""

from __future__ import annotations

from typing import Collection, Generator, Iterable, List, Tuple, TypeVar, Union

T = TypeVar("T")
K = TypeVar("K")
V = TypeVar("V")


__all__ = ["boundary_iter", "flatten", "as_list"]


def boundary_iter(obj: Iterable[T], /) -> Generator[Tuple[bool, T], None, None]:
    """
    Iterates over an iterable, returning a boolean and the underlying values indicating
    whether the iterator has reached its last item.

    >>> list(boundary_iter(range(0, 3)))
    [(False, 0), (False, 1), (True, 2)]

    >>> list(boundary_iter('A'))
    [(True, 'A')]

    # empty example
    >>> list(boundary_iter([]))
    []

    # iterating over an infinite iterable
    >>> import itertools
    >>> list(itertools.islice(boundary_iter(iter(lambda:0,1)), 4))
    [(False, 0), (False, 0), (False, 0), (False, 0)]
    """
    gen = iter(obj)
    prev_entry = None
    while True:
        try:
            entry = next(gen)
            if prev_entry is not None:
                yield False, prev_entry
            prev_entry = entry
        except StopIteration:
            if prev_entry is not None:
                yield True, prev_entry
            break


def flatten(obj, /):
    ret = []
    for sublist in obj:
        ret.extend(sublist)
    return ret


def as_list(obj: Union[None, T, Collection[Union[None, T]]], /) -> List[T]:
    """
    Convert an object that is either nothing, a single object, or a collection, to a list of type
    of that object.
    """
    from collections.abc import Iterable

    if obj is None:
        return []
    elif isinstance(obj, str):
        # strings are iterable, but it's almost never intended to be used in place of a list; if
        # we're given a string, then assume what is wanted is a single list containing the full
        # string instead of a list containing every character as an individual item
        return [obj]  # type: ignore
    elif isinstance(obj, Iterable):
        return [o for o in obj if o is not None]
    else:
        # assume we're dealing with a single object of the requested type
        return [obj]



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

import builtins as _builtins, sys, typing as _typing

from google.protobuf.internal.containers import RepeatedScalarFieldContainer
from google.protobuf.message import Message as _Message

from .event_pb2 import ArchivedEvent, CreatedEvent
from .value_pb2 import Identifier, Value

__all__ = [
    "GetEventsByContractIdRequest",
    "GetEventsByContractIdResponse",
    "GetEventsByContractKeyRequest",
    "GetEventsByContractKeyResponse",
]


class GetEventsByContractIdRequest(_Message):
    contract_id: _builtins.str
    @property
    def requesting_parties(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    def __init__(self, *, contract_id: _typing.Optional[_builtins.str] = ..., requesting_parties: _typing.Optional[_typing.Iterable[_builtins.str]] = ...): ...
    def HasField(self, field_name: _typing.Literal["contract_id", "requesting_parties"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["contract_id", "requesting_parties"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetEventsByContractIdResponse(_Message):
    @property
    def create_event(self) -> CreatedEvent: ...
    @property
    def archive_event(self) -> ArchivedEvent: ...
    def __init__(self, *, create_event: _typing.Optional[CreatedEvent] = ..., archive_event: _typing.Optional[ArchivedEvent] = ...): ...
    def HasField(self, field_name: _typing.Literal["create_event", "archive_event"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["create_event", "archive_event"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetEventsByContractKeyRequest(_Message):
    @property
    def contract_key(self) -> Value: ...
    @property
    def template_id(self) -> Identifier: ...
    @property
    def requesting_parties(self) -> RepeatedScalarFieldContainer[_builtins.str]: ...
    continuation_token: _builtins.str
    def __init__(self, *, contract_key: _typing.Optional[Value] = ..., template_id: _typing.Optional[Identifier] = ..., requesting_parties: _typing.Optional[_typing.Iterable[_builtins.str]] = ..., continuation_token: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["contract_key", "template_id", "requesting_parties", "continuation_token"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["contract_key", "template_id", "requesting_parties", "continuation_token"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...

class GetEventsByContractKeyResponse(_Message):
    @property
    def create_event(self) -> CreatedEvent: ...
    @property
    def archive_event(self) -> ArchivedEvent: ...
    continuation_token: _builtins.str
    def __init__(self, *, create_event: _typing.Optional[CreatedEvent] = ..., archive_event: _typing.Optional[ArchivedEvent] = ..., continuation_token: _typing.Optional[_builtins.str] = ...): ...
    def HasField(self, field_name: _typing.Literal["create_event", "archive_event", "continuation_token"]) -> _builtins.bool: ...
    def ClearField(self, field_name: _typing.Literal["create_event", "archive_event", "continuation_token"]) -> None: ...
    def WhichOneof(self, oneof_group: _typing.NoReturn) -> _typing.NoReturn: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import Any

from ..damlast.daml_lf_1 import TypeConName
from ..ledger import CreateAndExerciseCommand, CreateCommand, ExerciseByKeyCommand, ExerciseCommand
from ..prim import ContractId
from ..protocols.serializers import AbstractSerializer
from ..values import CanonicalMapper

__all__ = ["ValidateSerializer"]


class ValidateSerializer(AbstractSerializer):
    """
    A serializer that doesn't actually write to an on-wire form, but merely does lightweight
    conversion and give on-wire serialization the best chance of succeeding.

    This class essentially handles type conversion before a command is fully created and sent to
    a background thread for eventual writing to the Ledger API.
    """

    mapper = CanonicalMapper()

    def serialize_create_command(self, name: TypeConName, template_args: Any) -> CreateCommand:
        return CreateCommand(template_id=name, payload=template_args)

    def serialize_exercise_command(
        self, contract_id: ContractId, choice_name: str, choice_args: Any
    ) -> ExerciseCommand:
        return ExerciseCommand(contract_id=contract_id, choice=choice_name, argument=choice_args)

    def serialize_exercise_by_key_command(
        self,
        template_name: TypeConName,
        key_arguments: Any,
        choice_name: str,
        choice_arguments: Any,
    ) -> ExerciseByKeyCommand:
        return ExerciseByKeyCommand(
            template_id=template_name,
            key=key_arguments,
            choice=choice_name,
            argument=choice_arguments,
        )

    def serialize_create_and_exercise_command(
        self,
        template_name: TypeConName,
        create_arguments: Any,
        choice_name: str,
        choice_arguments: Any,
    ) -> CreateAndExerciseCommand:
        return CreateAndExerciseCommand(
            template_id=template_name,
            payload=create_arguments,
            choice=choice_name,
            argument=choice_arguments,
        )



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/experimental_features.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2com/daml/ledger/api/v1/experimental_features.proto\x12\x16\x63om.daml.ledger.api.v1\"\xd6\x07\n\x14\x45xperimentalFeatures\x12v\n\x18self_service_error_codes\x18\x01 \x01(\x0b\x32\x39.com.daml.ledger.api.v1.ExperimentalSelfServiceErrorCodesB\x02\x18\x01R\x15selfServiceErrorCodes\x12O\n\x0bstatic_time\x18\x02 \x01(\x0b\x32..com.daml.ledger.api.v1.ExperimentalStaticTimeR\nstaticTime\x12i\n\x15\x63ommand_deduplication\x18\x03 \x01(\x0b\x32\x34.com.daml.ledger.api.v1.CommandDeduplicationFeaturesR\x14\x63ommandDeduplication\x12\x62\n\x12optional_ledger_id\x18\x04 \x01(\x0b\x32\x34.com.daml.ledger.api.v1.ExperimentalOptionalLedgerIdR\x10optionalLedgerId\x12R\n\x0c\x63ontract_ids\x18\x05 \x01(\x0b\x32/.com.daml.ledger.api.v1.ExperimentalContractIdsR\x0b\x63ontractIds\x12\x65\n\x13\x63ommitter_event_log\x18\x06 \x01(\x0b\x32\x35.com.daml.ledger.api.v1.ExperimentalCommitterEventLogR\x11\x63ommitterEventLog\x12g\n\x13\x65xplicit_disclosure\x18\x07 \x01(\x0b\x32\x36.com.daml.ledger.api.v1.ExperimentalExplicitDisclosureR\x12\x65xplicitDisclosure\x12\x9e\x01\n(user_and_party_local_metadata_extensions\x18\x08 \x01(\x0b\x32G.com.daml.ledger.api.v1.ExperimentalUserAndPartyLocalMetadataExtensionsR#userAndPartyLocalMetadataExtensions\x12\x61\n\x14\x61\x63s_active_at_offset\x18\t \x01(\x0b\x32\x30.com.daml.ledger.api.v1.AcsActiveAtOffsetFeatureR\x11\x61\x63sActiveAtOffset\"\'\n!ExperimentalSelfServiceErrorCodes:\x02\x18\x01\"6\n\x16\x45xperimentalStaticTime\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"\xcb\x02\n\x1c\x43ommandDeduplicationFeatures\x12{\n\x1c\x64\x65\x64uplication_period_support\x18\x01 \x01(\x0b\x32\x39.com.daml.ledger.api.v1.CommandDeduplicationPeriodSupportR\x1a\x64\x65\x64uplicationPeriodSupport\x12_\n\x12\x64\x65\x64uplication_type\x18\x02 \x01(\x0e\x32\x30.com.daml.ledger.api.v1.CommandDeduplicationTypeR\x11\x64\x65\x64uplicationType\x12M\n#max_deduplication_duration_enforced\x18\x03 \x01(\x08R maxDeduplicationDurationEnforced\"\x1e\n\x1c\x45xperimentalOptionalLedgerId\"\xbf\x03\n!CommandDeduplicationPeriodSupport\x12n\n\x0eoffset_support\x18\x01 \x01(\x0e\x32G.com.daml.ledger.api.v1.CommandDeduplicationPeriodSupport.OffsetSupportR\roffsetSupport\x12t\n\x10\x64uration_support\x18\x02 \x01(\x0e\x32I.com.daml.ledger.api.v1.CommandDeduplicationPeriodSupport.DurationSupportR\x0f\x64urationSupport\"d\n\rOffsetSupport\x12\x18\n\x14OFFSET_NOT_SUPPORTED\x10\x00\x12\x19\n\x15OFFSET_NATIVE_SUPPORT\x10\x01\x12\x1e\n\x1aOFFSET_CONVERT_TO_DURATION\x10\x02\"N\n\x0f\x44urationSupport\x12\x1b\n\x17\x44URATION_NATIVE_SUPPORT\x10\x00\x12\x1e\n\x1a\x44URATION_CONVERT_TO_OFFSET\x10\x01\"\xa5\x01\n\x17\x45xperimentalContractIds\x12S\n\x02v1\x18\x01 \x01(\x0e\x32\x43.com.daml.ledger.api.v1.ExperimentalContractIds.ContractIdV1SupportR\x02v1\"5\n\x13\x43ontractIdV1Support\x12\x0c\n\x08SUFFIXED\x10\x00\x12\x10\n\x0cNON_SUFFIXED\x10\x01\"\xcd\x01\n\x1d\x45xperimentalCommitterEventLog\x12q\n\x0e\x65vent_log_type\x18\x01 \x01(\x0e\x32K.com.daml.ledger.api.v1.ExperimentalCommitterEventLog.CommitterEventLogTypeR\x0c\x65ventLogType\"9\n\x15\x43ommitterEventLogType\x12\x0f\n\x0b\x43\x45NTRALIZED\x10\x00\x12\x0f\n\x0b\x44ISTRIBUTED\x10\x01\">\n\x1e\x45xperimentalExplicitDisclosure\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"O\n/ExperimentalUserAndPartyLocalMetadataExtensions\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported\"8\n\x18\x41\x63sActiveAtOffsetFeature\x12\x1c\n\tsupported\x18\x01 \x01(\x08R\tsupported*I\n\x18\x43ommandDeduplicationType\x12\x0e\n\nASYNC_ONLY\x10\x00\x12\x1d\n\x19\x41SYNC_AND_CONCURRENT_SYNC\x10\x01\x42\x98\x01\n\x16\x63om.daml.ledger.api.v1B\x1e\x45xperimentalFeaturesOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.daml.ledger.api.v1.experimental_features_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\036ExperimentalFeaturesOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _EXPERIMENTALFEATURES.fields_by_name['self_service_error_codes']._options = None
  _EXPERIMENTALFEATURES.fields_by_name['self_service_error_codes']._serialized_options = b'\030\001'
  _EXPERIMENTALSELFSERVICEERRORCODES._options = None
  _EXPERIMENTALSELFSERVICEERRORCODES._serialized_options = b'\030\001'
  _globals['_COMMANDDEDUPLICATIONTYPE']._serialized_start=2555
  _globals['_COMMANDDEDUPLICATIONTYPE']._serialized_end=2628
  _globals['_EXPERIMENTALFEATURES']._serialized_start=79
  _globals['_EXPERIMENTALFEATURES']._serialized_end=1061
  _globals['_EXPERIMENTALSELFSERVICEERRORCODES']._serialized_start=1063
  _globals['_EXPERIMENTALSELFSERVICEERRORCODES']._serialized_end=1102
  _globals['_EXPERIMENTALSTATICTIME']._serialized_start=1104
  _globals['_EXPERIMENTALSTATICTIME']._serialized_end=1158
  _globals['_COMMANDDEDUPLICATIONFEATURES']._serialized_start=1161
  _globals['_COMMANDDEDUPLICATIONFEATURES']._serialized_end=1492
  _globals['_EXPERIMENTALOPTIONALLEDGERID']._serialized_start=1494
  _globals['_EXPERIMENTALOPTIONALLEDGERID']._serialized_end=1524
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT']._serialized_start=1527
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT']._serialized_end=1974
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_OFFSETSUPPORT']._serialized_start=1794
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_OFFSETSUPPORT']._serialized_end=1894
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_DURATIONSUPPORT']._serialized_start=1896
  _globals['_COMMANDDEDUPLICATIONPERIODSUPPORT_DURATIONSUPPORT']._serialized_end=1974
  _globals['_EXPERIMENTALCONTRACTIDS']._serialized_start=1977
  _globals['_EXPERIMENTALCONTRACTIDS']._serialized_end=2142
  _globals['_EXPERIMENTALCONTRACTIDS_CONTRACTIDV1SUPPORT']._serialized_start=2089
  _globals['_EXPERIMENTALCONTRACTIDS_CONTRACTIDV1SUPPORT']._serialized_end=2142
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG']._serialized_start=2145
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG']._serialized_end=2350
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG_COMMITTEREVENTLOGTYPE']._serialized_start=2293
  _globals['_EXPERIMENTALCOMMITTEREVENTLOG_COMMITTEREVENTLOGTYPE']._serialized_end=2350
  _globals['_EXPERIMENTALEXPLICITDISCLOSURE']._serialized_start=2352
  _globals['_EXPERIMENTALEXPLICITDISCLOSURE']._serialized_end=2414
  _globals['_EXPERIMENTALUSERANDPARTYLOCALMETADATAEXTENSIONS']._serialized_start=2416
  _globals['_EXPERIMENTALUSERANDPARTYLOCALMETADATAEXTENSIONS']._serialized_end=2495
  _globals['_ACSACTIVEATOFFSETFEATURE']._serialized_start=2497
  _globals['_ACSACTIVEATOFFSETFEATURE']._serialized_end=2553
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from .api import MetricEvents



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import abc
from datetime import datetime
from typing import (
    AbstractSet,
    Any,
    AsyncIterator,
    Awaitable,
    Callable,
    Collection,
    DefaultDict,
    List,
    Optional,
    Protocol,
    Sequence,
    TypeVar,
    Union,
    overload,
    runtime_checkable,
)

from .. import (
    Connection as _Connection,
    PackageService as _PackageService,
    QueryStream as _QueryStream,
)
from ...damlast.daml_lf_1 import PackageRef, TypeConName
from ...prim import ContractData, ContractId, Party, TimeDeltaLike
from ...query import Queries, Query
from ..api_types import (
    ArchiveEvent,
    Boundary,
    Command,
    CreateEvent,
    ExerciseResponse,
    MeteringReport,
    PartyInfo,
    Right,
    SubmitResponse,
    User,
    Version,
)
from .pkgloader import PackageLoader

__all__ = ["PackageService", "Connection", "QueryStream", "QueryStreamBase", "PackageLoader"]

ConnSelf = TypeVar("ConnSelf", bound="Connection")
QSSelf = TypeVar("QSSelf", bound="QueryStream")
CreateFn = TypeVar("CreateFn", bound=Callable[[CreateEvent], SubmitResponse])
ACreateFn = TypeVar("ACreateFn", bound=Callable[[CreateEvent], Awaitable[SubmitResponse]])
ArchiveFn = TypeVar("ArchiveFn", bound=Callable[[ArchiveEvent], SubmitResponse])
AArchiveFn = TypeVar("AArchiveFn", bound=Callable[[ArchiveEvent], Awaitable[SubmitResponse]])
BoundaryFn = TypeVar("BoundaryFn", bound=Callable[[Boundary], SubmitResponse])
ABoundaryFn = TypeVar("ABoundaryFn", bound=Callable[[Boundary], Awaitable[SubmitResponse]])

# mypy treats Callables as _covariant_ in their arguments, and _contravariant_ in their return
# types. That means that A -> A and B -> B are incompatible types.
#
# However, we can define an object that has two overloaded __call__ signatures, which means that
# we have an *object* that can be used like a function, but supports adding additional "overloads",
# such that the base type supports being called with A, and returns A, and the subtype supports
# that, AS WELL AS being called with B and returning B.
#
# mypy completely understands what is going on here, and manages to validate correct usages of these
# decorators, *as well as* flagging incorrect usages!
class ACreateDecorator(Protocol):
    @overload
    def __call__(self, fn: CreateFn, /) -> CreateFn: ...
    @overload
    def __call__(self, fn: ACreateFn, /) -> ACreateFn: ...

class AArchiveDecorator(Protocol):
    @overload
    def __call__(self, fn: ArchiveFn, /) -> ArchiveFn: ...
    @overload
    def __call__(self, fn: AArchiveFn, /) -> AArchiveFn: ...

class ABoundaryDecorator(Protocol):
    @overload
    def __call__(self, fn: BoundaryFn, /) -> BoundaryFn: ...
    @overload
    def __call__(self, fn: ABoundaryFn, /) -> ABoundaryFn: ...

class PackageService(_PackageService, Protocol):
    async def get_package(
        self, package_id: PackageRef, /, *, timeout: Optional[TimeDeltaLike] = ...
    ) -> bytes: ...
    async def list_package_ids(
        self, *, timeout: Optional[TimeDeltaLike] = ...
    ) -> AbstractSet[PackageRef]: ...

@runtime_checkable
class Connection(_Connection, PackageService, Protocol):
    async def __aenter__(self: ConnSelf) -> ConnSelf: ...
    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None: ...
    async def open(self) -> None: ...
    async def close(self) -> None: ...
    async def submit(
        self,
        commands: Union[Command, Sequence[Command]],
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Union[None, Party, Collection[Party]] = None,
        act_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> None: ...
    async def create(
        self,
        template_id: Union[str, TypeConName],
        payload: ContractData,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Union[None, Party, Collection[Party]] = None,
        act_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> CreateEvent: ...
    async def exercise(
        self,
        contract_id: ContractId,
        choice_name: str,
        argument: Optional[ContractData] = None,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Union[None, Party, Collection[Party]] = None,
        act_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> ExerciseResponse: ...
    async def create_and_exercise(
        self,
        template_id: Union[str, TypeConName],
        payload: ContractData,
        choice_name: str,
        argument: Optional[ContractData] = None,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Union[None, Party, Collection[Party]] = None,
        act_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> ExerciseResponse: ...
    async def exercise_by_key(
        self,
        template_id: Union[str, TypeConName],
        choice_name: str,
        key: Any,
        argument: Optional[ContractData] = None,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Union[None, Party, Collection[Party]] = None,
        act_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> ExerciseResponse: ...
    async def archive(
        self,
        contract_id: ContractId,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Union[None, Party, Collection[Party]] = None,
        act_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> ArchiveEvent: ...
    async def archive_by_key(
        self,
        template_id: str,
        key: Any,
        /,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
        read_as: Union[None, Party, Collection[Party]] = None,
        act_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> ArchiveEvent: ...
    async def get_ledger_end(self, *, timeout: Optional[TimeDeltaLike] = ...) -> str: ...
    def query(
        self,
        template_id: Union[str, TypeConName] = "*",
        query: Query = None,
        /,
        *,
        read_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> QueryStream: ...
    def query_many(
        self,
        *queries: Queries,
        read_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> QueryStream: ...
    def stream(
        self,
        template_id: Union[str, TypeConName] = "*",
        query: Query = None,
        /,
        *,
        offset: Optional[str] = None,
        read_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> QueryStream: ...
    def stream_many(
        self,
        *queries: Queries,
        offset: Optional[str] = None,
        read_as: Union[None, Party, Collection[Party]] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> QueryStream: ...
    async def get_user(
        self, user_id: Optional[str] = None, /, *, timeout: Optional[TimeDeltaLike] = ...
    ) -> User: ...
    async def create_user(
        self,
        user: User,
        rights: Optional[Sequence[Right]] = ...,
        *,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> User: ...
    async def list_users(self, *, timeout: Optional[TimeDeltaLike] = ...) -> Sequence[User]: ...
    async def list_user_rights(
        self, user_id: Optional[str] = None, /, *, timeout: Optional[TimeDeltaLike] = ...
    ) -> Sequence[Right]: ...
    async def allocate_party(
        self,
        *,
        identifier_hint: Optional[str] = None,
        display_name: Optional[str] = None,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> PartyInfo: ...
    async def list_known_parties(
        self, *, timeout: Optional[TimeDeltaLike] = ...
    ) -> Sequence[PartyInfo]: ...
    async def get_version(self, *, timeout: Optional[TimeDeltaLike] = ...) -> Version: ...
    async def upload_package(
        self, contents: bytes, /, *, timeout: Optional[TimeDeltaLike] = ...
    ) -> None: ...
    async def get_metering_report(
        self,
        from_: datetime,
        to: Optional[datetime] = None,
        application_id: Optional[str] = None,
        *,
        timeout: Optional[TimeDeltaLike] = ...,
    ) -> MeteringReport: ...

# PyCharm doesn't know what to make of these overloads with respect to the parent protocol,
# but mypy understands that these type signatures do not conflict with the parent base class
# noinspection PyProtocol,PyMethodOverriding
@runtime_checkable
class QueryStream(_QueryStream, Protocol):
    @overload
    def on_create(self) -> ACreateDecorator: ...
    @overload
    def on_create(self, fn: CreateFn, /) -> CreateFn: ...
    @overload
    def on_create(self, fn: ACreateFn, /) -> ACreateFn: ...
    @overload
    def on_create(self, name: Union[str, TypeConName], /) -> ACreateDecorator: ...
    @overload
    def on_create(self, name: Union[str, TypeConName], fn: CreateFn, /) -> CreateFn: ...
    @overload
    def on_create(self, name: Union[str, TypeConName], fn: ACreateFn, /) -> ACreateFn: ...
    @overload
    def on_archive(self) -> AArchiveDecorator: ...
    @overload
    def on_archive(self, fn: ArchiveFn, /) -> ArchiveFn: ...
    @overload
    def on_archive(self, fn: AArchiveFn, /) -> AArchiveFn: ...
    @overload
    def on_archive(self, name: Union[str, TypeConName], /) -> AArchiveDecorator: ...
    @overload
    def on_archive(self, name: Union[str, TypeConName], fn: ArchiveFn, /) -> ArchiveFn: ...
    @overload
    def on_archive(self, name: Union[str, TypeConName], fn: AArchiveFn, /) -> AArchiveFn: ...
    @overload
    def on_boundary(self) -> ABoundaryDecorator: ...
    @overload
    def on_boundary(self, fn: BoundaryFn, /) -> BoundaryFn: ...
    @overload
    def on_boundary(self, fn: ABoundaryFn, /) -> ABoundaryFn: ...
    def creates(self) -> AsyncIterator[CreateEvent]: ...
    def events(self) -> AsyncIterator[Union[CreateEvent, ArchiveEvent]]: ...
    def items(self) -> AsyncIterator[Union[CreateEvent, ArchiveEvent, Boundary]]: ...
    def __aiter__(self) -> AsyncIterator[Union[CreateEvent, ArchiveEvent, Boundary]]: ...
    async def run(self) -> None: ...
    async def close(self) -> None: ...
    async def __aenter__(self: QSSelf) -> QSSelf: ...
    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None: ...

class QueryStreamBase(QueryStream, abc.ABC):
    @property
    def _callbacks(self) -> DefaultDict[str, List[Callable]]: ...
    @abc.abstractmethod
    def items(self): ...
    async def _emit(self, name: str, obj: Any, /) -> None: ...
    async def _emit_create(self, event: CreateEvent, /) -> None: ...
    async def _emit_archive(self, event: ArchiveEvent, /) -> None: ...
    async def _emit_boundary(self, event: Boundary, /) -> None: ...



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Types that describe the behavior of the ledger itself.
"""
from __future__ import annotations

from dataclasses import dataclass

from ..ledger.aio import PackageLoader
from ..protocols.serializers import Serializer

__all__ = ["LedgerMetadata"]


@dataclass(init=False, frozen=True)
class LedgerMetadata:
    """
    Attributes that are invariant with respect to any party on the ledger.
    """

    ledger_id: str
    package_loader: PackageLoader
    serializer: Serializer
    protocol_version: str

    def __init__(
        self,
        ledger_id: str,
        package_loader: PackageLoader,
        serializer: Serializer,
        protocol_version: str,
    ):
        object.__setattr__(self, "ledger_id", ledger_id)
        object.__setattr__(self, "package_loader", package_loader)
        object.__setattr__(self, "serializer", serializer)
        object.__setattr__(self, "protocol_version", protocol_version)



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/event_query_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import event_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__pb2
from . import value_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0com/daml/ledger/api/v1/event_query_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a\"com/daml/ledger/api/v1/event.proto\x1a\"com/daml/ledger/api/v1/value.proto\"n\n\x1cGetEventsByContractIdRequest\x12\x1f\n\x0b\x63ontract_id\x18\x01 \x01(\tR\ncontractId\x12-\n\x12requesting_parties\x18\x02 \x03(\tR\x11requestingParties\"\xb4\x01\n\x1dGetEventsByContractIdResponse\x12G\n\x0c\x63reate_event\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.CreatedEventR\x0b\x63reateEvent\x12J\n\rarchive_event\x18\x02 \x01(\x0b\x32%.com.daml.ledger.api.v1.ArchivedEventR\x0c\x61rchiveEvent\"\x84\x02\n\x1dGetEventsByContractKeyRequest\x12@\n\x0c\x63ontract_key\x18\x01 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.ValueR\x0b\x63ontractKey\x12\x43\n\x0btemplate_id\x18\x02 \x01(\x0b\x32\".com.daml.ledger.api.v1.IdentifierR\ntemplateId\x12-\n\x12requesting_parties\x18\x03 \x03(\tR\x11requestingParties\x12-\n\x12\x63ontinuation_token\x18\x04 \x01(\tR\x11\x63ontinuationToken\"\xe4\x01\n\x1eGetEventsByContractKeyResponse\x12G\n\x0c\x63reate_event\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.CreatedEventR\x0b\x63reateEvent\x12J\n\rarchive_event\x18\x02 \x01(\x0b\x32%.com.daml.ledger.api.v1.ArchivedEventR\x0c\x61rchiveEvent\x12-\n\x12\x63ontinuation_token\x18\x04 \x01(\tR\x11\x63ontinuationToken2\xa4\x02\n\x11\x45ventQueryService\x12\x84\x01\n\x15GetEventsByContractId\x12\x34.com.daml.ledger.api.v1.GetEventsByContractIdRequest\x1a\x35.com.daml.ledger.api.v1.GetEventsByContractIdResponse\x12\x87\x01\n\x16GetEventsByContractKey\x12\x35.com.daml.ledger.api.v1.GetEventsByContractKeyRequest\x1a\x36.com.daml.ledger.api.v1.GetEventsByContractKeyResponseB\x95\x01\n\x16\x63om.daml.ledger.api.v1B\x1b\x45ventQueryServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.daml.ledger.api.v1.event_query_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.daml.ledger.api.v1B\033EventQueryServiceOuterClassZEgithub.com/digital-asset/dazl-client/v7/go/api/com/daml/ledger/api/v1\252\002\026Com.Daml.Ledger.Api.V1'
  _globals['_GETEVENTSBYCONTRACTIDREQUEST']._serialized_start=148
  _globals['_GETEVENTSBYCONTRACTIDREQUEST']._serialized_end=258
  _globals['_GETEVENTSBYCONTRACTIDRESPONSE']._serialized_start=261
  _globals['_GETEVENTSBYCONTRACTIDRESPONSE']._serialized_end=441
  _globals['_GETEVENTSBYCONTRACTKEYREQUEST']._serialized_start=444
  _globals['_GETEVENTSBYCONTRACTKEYREQUEST']._serialized_end=704
  _globals['_GETEVENTSBYCONTRACTKEYRESPONSE']._serialized_start=707
  _globals['_GETEVENTSBYCONTRACTKEYRESPONSE']._serialized_end=935
  _globals['_EVENTQUERYSERVICE']._serialized_start=938
  _globals['_EVENTQUERYSERVICE']._serialized_end=1230
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import Generic, Optional, TypeVar

from .daml_lf_1 import (
    Binding,
    Block,
    BuiltinFunction,
    Case,
    Expr,
    FieldWithExpr,
    FieldWithType,
    Module,
    Package,
    PrimCon,
    PrimLit,
    Scenario,
    Type,
    Update,
    ValName,
    VarWithType,
)

E = TypeVar("E", contravariant=True)
RP = TypeVar("RP", covariant=True)
RM = TypeVar("RM", covariant=True)
RE = TypeVar("RE", covariant=True)
RT = TypeVar("RT", covariant=True)


__all__ = [
    "ExprVisitor",
    "IdentityTypeVisitor",
    "IdentityVisitor",
    "ModuleVisitor",
    "PackageVisitor",
    "TypeVisitor",
]


class PackageVisitor(Generic[RP]):
    def visit_package(self, p: Package, /) -> RP:
        raise NotImplementedError


class ModuleVisitor(Generic[RM]):
    def visit_module(self, m: Module, /) -> RM:
        raise NotImplementedError


class ExprVisitor(Generic[RE]):
    def visit_expr(self, e: Expr, /) -> RE:
        return e.Sum_match(
            self.visit_expr_var,
            self.visit_expr_val,
            self.visit_expr_builtin,
            self.visit_expr_prim_con,
            self.visit_expr_prim_lit,
            self.visit_expr_rec_con,
            self.visit_expr_rec_proj,
            self.visit_expr_rec_upd,
            self.visit_expr_variant_con,
            self.visit_expr_enum_con,
            self.visit_expr_struct_con,
            self.visit_expr_struct_proj,
            self.visit_expr_struct_upd,
            self.visit_expr_app,
            self.visit_expr_ty_app,
            self.visit_expr_abs,
            self.visit_expr_ty_abs,
            self.visit_expr_case,
            self.visit_expr_let,
            self.visit_expr_nil,
            self.visit_expr_cons,
            self.visit_expr_update,
            self.visit_expr_scenario,
            self.visit_expr_optional_none,
            self.visit_expr_optional_some,
            self.visit_expr_to_any,
            self.visit_expr_from_any,
            self.visit_expr_type_rep,
            self.visit_expr_to_any_exception,
            self.visit_expr_from_any_exception,
            self.visit_expr_throw,
            self.visit_expr_to_interface,
            self.visit_expr_from_interface,
            self.visit_expr_call_interface,
            self.visit_expr_signatory_interface,
            self.visit_expr_observer_interface,
            self.visit_expr_view_interface,
            self.visit_expr_unsafe_from_interface,
            self.visit_expr_interface_template_type_rep,
            self.visit_expr_to_required_interface,
            self.visit_expr_from_required_interface,
            self.visit_expr_unsafe_from_required_interface,
            self.visit_expr_experimental,
        )

    def visit_expr_var(self, e: str, /) -> RE:
        raise NotImplementedError

    def visit_expr_val(self, e: ValName, /) -> RE:
        raise NotImplementedError

    def visit_expr_builtin(self, e: BuiltinFunction, /) -> RE:
        raise NotImplementedError

    def visit_expr_prim_con(self, e: PrimCon, /) -> RE:
        raise NotImplementedError

    def visit_expr_prim_lit(self, e: PrimLit, /) -> RE:
        raise NotImplementedError

    def visit_expr_rec_con(self, e: Expr.RecCon, /) -> RE:
        raise NotImplementedError

    def visit_expr_rec_proj(self, e: Expr.RecProj, /) -> RE:
        raise NotImplementedError

    def visit_expr_variant_con(self, e: Expr.VariantCon, /) -> RE:
        raise NotImplementedError

    def visit_expr_enum_con(self, e: Expr.EnumCon, /) -> RE:
        raise NotImplementedError

    def visit_expr_struct_con(self, e: Expr.StructCon, /) -> RE:
        raise NotImplementedError

    def visit_expr_struct_proj(self, e: Expr.StructProj, /) -> RE:
        raise NotImplementedError

    def visit_expr_app(self, e: Expr.App, /) -> RE:
        raise NotImplementedError

    def visit_expr_ty_app(self, e: Expr.TyApp, /) -> RE:
        raise NotImplementedError

    def visit_expr_abs(self, e: Expr.Abs, /) -> RE:
        raise NotImplementedError

    def visit_expr_ty_abs(self, e: Expr.TyAbs, /) -> RE:
        raise NotImplementedError

    def visit_expr_case(self, e: Case, /) -> RE:
        raise NotImplementedError

    def visit_expr_let(self, e: Block, /) -> RE:
        raise NotImplementedError

    def visit_expr_nil(self, e: Expr.Nil, /) -> RE:
        raise NotImplementedError

    def visit_expr_cons(self, e: Expr.Cons, /) -> RE:
        raise NotImplementedError

    def visit_expr_update(self, e: Update, /) -> RE:
        raise NotImplementedError

    def visit_expr_scenario(self, e: Scenario, /) -> RE:
        raise NotImplementedError

    def visit_expr_rec_upd(self, e: Expr.RecUpd, /) -> RE:
        raise NotImplementedError

    def visit_expr_struct_upd(self, e: Expr.StructUpd, /) -> RE:
        raise NotImplementedError

    def visit_expr_optional_none(self, e: Expr.OptionalNone, /) -> RE:
        raise NotImplementedError

    def visit_expr_optional_some(self, e: Expr.OptionalSome, /) -> RE:
        raise NotImplementedError

    def visit_expr_to_any(self, e: Expr.ToAny, /) -> RE:
        raise NotImplementedError

    def visit_expr_from_any(self, e: Expr.FromAny, /) -> RE:
        raise NotImplementedError

    def visit_expr_type_rep(self, e: Type, /) -> RE:
        raise NotImplementedError

    def visit_expr_throw(self, e: Expr.Throw, /) -> RE:
        raise NotImplementedError

    def visit_expr_to_any_exception(self, e: Expr.ToAnyException, /) -> RE:
        raise NotImplementedError

    def visit_expr_from_any_exception(self, e: Expr.FromAnyException, /) -> RE:
        raise NotImplementedError

    def visit_expr_to_interface(self, e: Expr.ToInterface, /) -> RE:
        raise NotImplementedError

    def visit_expr_from_interface(self, e: Expr.FromInterface, /) -> RE:
        raise NotImplementedError

    def visit_expr_call_interface(self, e: Expr.CallInterface, /) -> RE:
        raise NotImplementedError

    def visit_expr_signatory_interface(self, e: Expr.SignatoryInterface, /) -> RE:
        raise NotImplementedError

    def visit_expr_observer_interface(self, e: Expr.ObserverInterface, /) -> RE:
        raise NotImplementedError

    def visit_expr_view_interface(self, e: Expr.ViewInterface, /) -> RE:
        raise NotImplementedError

    def visit_expr_unsafe_from_interface(self, e: Expr.UnsafeFromInterface, /) -> RE:
        raise NotImplementedError

    def visit_expr_interface_template_type_rep(self, e: Expr.InterfaceTemplateTypeRep, /) -> RE:
        raise NotImplementedError

    def visit_expr_to_required_interface(self, e: Expr.ToRequiredInterface, /) -> RE:
        raise NotImplementedError

    def visit_expr_from_required_interface(self, e: Expr.FromRequiredInterface, /) -> RE:
        raise NotImplementedError

    def visit_expr_unsafe_from_required_interface(
        self, e: Expr.UnsafeFromRequiredInterface, /
    ) -> RE:
        raise NotImplementedError

    def visit_expr_experimental(self, e: Expr.Experimental, /) -> RE:
        raise NotImplementedError


class TypeVisitor(Generic[RT]):
    def visit_type(self, t: Type, /) -> RT:
        return t.Sum_match(
            self.visit_type_var,
            self.visit_type_con,
            self.visit_type_prim,
            self.visit_type_forall,
            self.visit_type_struct,
            self.visit_type_nat,
            self.visit_type_syn,
        )

    def visit_type_var(self, t: Type.Var, /) -> RT:
        raise NotImplementedError

    def visit_type_con(self, t: Type.Con, /) -> RT:
        raise NotImplementedError

    def visit_type_prim(self, t: Type.Prim, /) -> RT:
        raise NotImplementedError

    def visit_type_forall(self, t: Type.Forall, /) -> RT:
        raise NotImplementedError

    def visit_type_struct(self, t: Type.Struct, /) -> RT:
        raise NotImplementedError

    def visit_type_nat(self, nat: int) -> RT:
        raise NotImplementedError

    def visit_type_syn(self, t: Type.Syn) -> RT:
        raise NotImplementedError


class IdentityTypeVisitor(TypeVisitor[Type]):
    def resolve_type(self, var: str, /) -> Optional[Type]:
        """
        Return a :class:`Type` for the specified variable, or ``None`` if there is no mapping for
        that type variable.
        """
        raise NotImplementedError

    def visit_type_var(self, t: Type.Var, /) -> Type:
        resolved_type = self.resolve_type(t.var)
        if resolved_type is not None:
            base_type = self.visit_type(resolved_type)
        else:
            base_type = Type(var=t)

        if t.args:
            new_args = tuple(self.visit_type(arg) for arg in t.args)
            return Type(var=Type.Var(var=t.var, args=new_args))
        else:
            return base_type

    def visit_type_con(self, t: Type.Con, /) -> Type:
        new_args = tuple(self.visit_type(arg) for arg in t.args)
        return Type(con=Type.Con(tycon=t.tycon, args=new_args))

    def visit_type_prim(self, t: Type.Prim, /) -> Type:
        new_args = tuple(self.visit_type(arg) for arg in t.args)
        return Type(prim=Type.Prim(prim=t.prim, args=new_args))

    def visit_type_forall(self, t: Type.Forall, /) -> Type:
        new_body = self.visit_type(t.body)
        return Type(forall=Type.Forall(t.vars, new_body))

    def visit_type_struct(self, t: Type.Struct, /) -> Type:
        new_fields = tuple(
            FieldWithType(field=fwt.field, type=self.visit_type(fwt.type)) for fwt in t.fields
        )
        return Type(struct=Type.Struct(fields=new_fields))

    def visit_type_nat(self, nat: int, /) -> Type:
        return Type(nat=nat)


class IdentityVisitor(ExprVisitor[Expr], IdentityTypeVisitor):
    """
    Implementation of :class:`ExprVisitor` that returns the same object for every case.
    """

    def visit_expr(self, e: Expr) -> Expr:
        return super(IdentityVisitor, self).visit_expr(e)

    def visit_expr_var(self, e: str, /) -> Expr:
        return Expr(var=e)

    def visit_expr_val(self, e: ValName, /) -> Expr:
        return Expr(val=e)

    def visit_expr_builtin(self, e: BuiltinFunction, /) -> Expr:
        return Expr(builtin=e)

    def visit_expr_prim_con(self, e: PrimCon, /) -> Expr:
        return Expr(prim_con=e)

    def visit_expr_prim_lit(self, e: PrimLit, /) -> Expr:
        return Expr(prim_lit=e)

    def visit_expr_rec_con(self, e: Expr.RecCon, /) -> Expr:
        new_fields = tuple(FieldWithExpr(fwt.field, self.visit_expr(fwt.expr)) for fwt in e.fields)
        new_tycon = self.visit_type_con(e.tycon).con
        return Expr(rec_con=Expr.RecCon(tycon=new_tycon, fields=new_fields))

    def visit_expr_rec_proj(self, e: Expr.RecProj, /) -> Expr:
        new_tycon = self.visit_type_con(e.tycon).con
        new_record = self.visit_expr(e.record)
        return Expr(rec_proj=Expr.RecProj(tycon=new_tycon, field=e.field, record=new_record))

    def visit_expr_variant_con(self, e: Expr.VariantCon, /) -> Expr:
        new_type_con = self.visit_type_con(e.tycon).con
        new_variant_arg = self.visit_expr(e.variant_arg)
        return Expr(
            variant_con=Expr.VariantCon(
                tycon=new_type_con, variant_con=e.variant_con, variant_arg=new_variant_arg
            )
        )

    def visit_expr_enum_con(self, e: Expr.EnumCon, /) -> Expr:
        return Expr(enum_con=Expr.EnumCon(tycon=e.tycon, enum_con=e.enum_con))

    def visit_expr_struct_con(self, e: Expr.StructCon, /) -> Expr:
        new_fields = tuple(FieldWithExpr(fwt.field, self.visit_expr(fwt.expr)) for fwt in e.fields)
        return Expr(struct_con=Expr.StructCon(fields=new_fields))

    def visit_expr_struct_proj(self, e: Expr.StructProj, /) -> Expr:
        new_struct = self.visit_expr(e.struct)
        return Expr(struct_proj=Expr.StructProj(field=e.field, struct=new_struct))

    def visit_expr_app(self, e: Expr.App, /) -> Expr:
        new_fun = self.visit_expr(e.fun)
        new_args = tuple([self.visit_expr(arg) for arg in e.args])
        return Expr(app=Expr.App(new_fun, new_args))

    def visit_expr_ty_app(self, e: Expr.TyApp, /) -> Expr:
        new_expr = self.visit_expr(e.expr)
        new_types = tuple(self.visit_type(t) for t in e.types)
        return Expr(ty_app=Expr.TyApp(new_expr, new_types))

    def visit_expr_abs(self, e: Expr.Abs, /) -> Expr:
        new_body = self.visit_expr(e.body)
        return Expr(abs=Expr.Abs(param=e.param, body=new_body))

    def visit_expr_ty_abs(self, e: Expr.TyAbs, /) -> Expr:
        new_body = self.visit_expr(e.body)
        return Expr(ty_abs=Expr.TyAbs(param=e.param, body=new_body))

    def visit_expr_case(self, e: Case, /) -> Expr:
        new_scrut = self.visit_expr(e.scrut)
        # TODO: Real implementation
        new_alts = [alt for alt in e.alts]
        return Expr(case=Case(scrut=new_scrut, alts=new_alts))

    def visit_expr_let(self, e: Block, /) -> Expr:
        new_bindings = [
            Binding(
                VarWithType(binding.binder.var, self.visit_type(binding.binder.type)),
                self.visit_expr(binding.bound),
            )
            for binding in e.bindings
        ]
        new_body = self.visit_expr(e.body)
        return Expr(let=Block(bindings=new_bindings, body=new_body))

    def visit_expr_nil(self, e: Expr.Nil, /) -> Expr:
        return Expr(nil=Expr.Nil(self.visit_type(e.type)))

    def visit_expr_cons(self, e: Expr.Cons, /) -> Expr:
        new_type = self.visit_type(e.type)
        new_front = tuple(self.visit_expr(front) for front in e.front)
        new_tail = self.visit_expr(e.tail)
        return Expr(cons=Expr.Cons(type=new_type, front=new_front, tail=new_tail))

    def visit_expr_update(self, e: Update, /) -> Expr:
        return Expr(update=e)

    def visit_expr_scenario(self, e: Scenario, /) -> Expr:
        return Expr(scenario=e)

    def visit_expr_rec_upd(self, e: Expr.RecUpd, /) -> Expr:
        return Expr(rec_upd=e)

    def visit_expr_struct_upd(self, e: Expr.StructUpd, /) -> Expr:
        return Expr(struct_upd=e)

    def visit_expr_optional_none(self, e: Expr.OptionalNone, /) -> Expr:
        new_type = self.visit_type(e.type)
        return Expr(optional_none=Expr.OptionalNone(type=new_type))

    def visit_expr_optional_some(self, e: Expr.OptionalSome, /) -> Expr:
        new_type = self.visit_type(e.type)
        new_body = self.visit_expr(e.body)
        return Expr(optional_some=Expr.OptionalSome(type=new_type, body=new_body))

    def visit_expr_to_any(self, e: Expr.ToAny, /) -> Expr:
        return Expr(to_any=Expr.ToAny(type=self.visit_type(e.type), expr=self.visit_expr(e.expr)))

    def visit_expr_from_any(self, e: Expr.FromAny, /) -> Expr:
        return Expr(
            from_any=Expr.FromAny(type=self.visit_type(e.type), expr=self.visit_expr(e.expr))
        )



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import event_query_service_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2


class EventQueryServiceStub(object):
    """Query events by contract id or key.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetEventsByContractId = channel.unary_unary(
                '/com.daml.ledger.api.v1.EventQueryService/GetEventsByContractId',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractIdRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractIdResponse.FromString,
                )
        self.GetEventsByContractKey = channel.unary_unary(
                '/com.daml.ledger.api.v1.EventQueryService/GetEventsByContractKey',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractKeyRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractKeyResponse.FromString,
                )


class EventQueryServiceServicer(object):
    """Query events by contract id or key.
    """

    def GetEventsByContractId(self, request, context):
        """Get the create and the consuming exercise event for the contract with the provided ID.
        No events will be returned for contracts that have been pruned because they 
        have already been archived before the latest pruning offset. 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetEventsByContractKey(self, request, context):
        """Get all create and consuming exercise events for the contracts with the provided contract key.
        Only events for unpruned contracts will be returned.
        Matching events are delivered in reverse chronological order, i.e.,
        the most recent events are delivered first.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_EventQueryServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetEventsByContractId': grpc.unary_unary_rpc_method_handler(
                    servicer.GetEventsByContractId,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractIdRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractIdResponse.SerializeToString,
            ),
            'GetEventsByContractKey': grpc.unary_unary_rpc_method_handler(
                    servicer.GetEventsByContractKey,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractKeyRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractKeyResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'com.daml.ledger.api.v1.EventQueryService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class EventQueryService(object):
    """Query events by contract id or key.
    """

    @staticmethod
    def GetEventsByContractId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.EventQueryService/GetEventsByContractId',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractIdRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractIdResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetEventsByContractKey(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/com.daml.ledger.api.v1.EventQueryService/GetEventsByContractKey',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractKeyRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__query__service__pb2.GetEventsByContractKeyResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright (c) 2017-2023 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import Optional


def n_things(count: int, plural_noun: str, singular_noun: Optional[str] = None):
    if count == 1:
        if singular_noun is None:
            return f"1 {plural_noun[0:-1]}"
        else:
            return f"1 {singular_noun}"
    else:
        return f"{count} {plural_noun}"
